
<?php

// WARNING:  This class ASSUMES you have a Progress supplied 
// ubroker.properties file for version 9+.  You cannot create a
// progress ubroker.properties file with this class, only manipulate an
// existing one.

class MngUBrokerProperties {

  // Variables available in this class

   var $FileText;     // This will be saved
   var $WorkingText;  // Used for moving around text, then sent to FileText.

  // This function loads the file into a big array that will be
  // processed into it's individual components later on.

  function Load ($FileName) {

     $fd = fopen ($FileName, "r");

     // The trim() below allows the in_array to work, but we do
     // do loose the leading spaces on some of the lines in the
     // file - so we are not writing back exactly in the same 
     // format.

     $iIter = 0;
     while (!feof($fd)) {
       $this->FileText [$iIter] = trim(fgets($fd));
       $iIter++;
     }

     fclose ($fd);
  } // function Load ()

  // This function creates a back up of the given file.  The file name is
  // postfixed with yyyydddhhmmss so it is good for a second's resolution.
  
  function Backup ($FileName) {
  
    if (!file_exists($FileName)) return;
	
    $Now = getdate();
	$PostFix = $Now["year"] . $Now["yday"] . $Now["days"] . $Now["hours"] . $Now["minutes"] . $Now["seconds"];
	$ToFileName = $FileName . "." . $PostFix;
	copy($FileName, $ToFileName);
	
  } // function Backup ()
  
  // This function will assemble the components into a new
  // ubroker.properties file.

  function Save ($FileName) {
  
    $this->Backup($FileName);

    $fd = fopen ($FileName, "w");

    $MaxIter = count ($this->FileText);
    for ($iIter = 0; $iIter < $MaxIter; $iIter++)
      fputs ($fd, $this->FileText[$iIter] . "\n");

    fclose($fd);

  } // function Save ()

  function GetDefinition ($DefName) {

    $DefName = "[UBroker.WS." . $DefName . "]";
    $CountOfLines = count($this->FileText);
    $InDelDefition = false;
    $WorkingiIter = 0;
    for($iIter = 0; $iIter < $CountOfLines; $iIter++) {

      $CurLine = $this->FileText[$iIter];

      // Have we encountered a definition line?

      if (strncmp($CurLine, "[", 1) == 0)
        if (strcmp($CurLine, $DefName) == 0) {
          $InDefinition = true;  // We found it!
          continue; // Skip past the [Ubroker... stuff
        }
        
      if (!$InDefinition) continue;

      // Encountered another definition, lets leave - no more to do

      if (strncmp($CurLine, "[", 1) == 0 && $InDefinition) break;

      if ($CurLine == "") continue;

      $LineDef=explode("=", $CurLine);
      $Definition[$LineDef[0]]=$LineDef[1];

    } // for()

    return $Definition;

  } // function GetDefinition ()

  // Add a non-existing WS broker definition to the file

  function AddDefinition ($DefName, $ArrayOfOptions) {

    // Locate the default parent values, then when we encounter
    // the next definition, we start putting our stuff in, then
    // continue with the next definition.  Hopefully, this will
    // keep like definitions amongst like definitions. 

    $DefName = "[UBroker.WS." . $DefName . "]";
    $CountOfLines = count($this->FileText);
    $FoundParentDefinition = false;
    $WorkingiIter = 0;

    for($iIter = 0; $iIter < $CountOfLines; $iIter++) {

      $CurLine = $this->FileText[$iIter];

      // Have we encountered a definition line?  And if so,
      // is it the parent defaults for Webspeed?

      if (strncmp($CurLine, "[", 1) == 0) {

        if (!$FoundParentDefinition && strcmp($CurLine, "[UBroker.WS]") == 0)
          $FoundParentDefinition = true;

        // We have found the parent definition, and it looks like
        // we are at the next definition, so lets slip our goods in
        // in here.

        else if ($FoundParentDefinition) {

          $this->WorkingText[$WorkingiIter] = $DefName;
          $WorkingiIter++;

          foreach ($ArrayOfOptions as $key => $Value) {
		    if ($key == "") continue;
            $this->WorkingText[$WorkingiIter] = $key
                                              . "=" 
                                              . $Value;
            $WorkingiIter++;

          }

          $FoundParentDefinition = false; // Should never get to true again
          $this->WorkingText[$WorkingiIter] = "";
          $WorkingiIter++;

        }

      }

      $this->WorkingText[$WorkingiIter] = $this->FileText[$iIter];
      $WorkingiIter++;

    } // for()

    unset($this->FileText);
    $this->FileText = $this->WorkingText;

  }

  // Here we delete the original definition and then simply 
  // add it.

  function ModDefinition ($DefName, $ArrayOfOptions) {
    $this->DelDefinition($DefName);
    $this->AddDefinition($DefName, $ArrayOfOptions);
  }

  // Here we basically populate $WorkingText until we encounter
  // the definition in question.  We then skip those lines until
  // the next definition. Then we reassign the variables.

  function DelDefinition ($DefName) {

    $DefName = "[UBroker.WS." . $DefName . "]";
    $CountOfLines = count($this->FileText);
    $InDelDefition = false;
    $WorkingiIter = 0;
    for($iIter = 0; $iIter < $CountOfLines; $iIter++) {

      $CurLine = $this->FileText[$iIter];

      // Have we encountered a definition line?

      if (strncmp($CurLine, "[", 1) == 0) 
        if (strcmp($CurLine, $DefName) == 0) $InDelDefinition = true;
        else $InDelDefinition = false;

      if ($InDelDefinition) continue;

      $this->WorkingText[$WorkingiIter] = $this->FileText[$iIter];
      $WorkingiIter++;

    } // for()

    unset($this->FileText);
    $this->FileText = $this->WorkingText;
  }

  function SetDefinition ($DefName, $ArrayOfOptions) {

    // Determine if we are adding or modifying
    // in_array would be better, but can't get it to work

    $TestDefName = "[UBroker.WS." . $DefName . "]";
    if (in_array($TestDefName, $this->FileText)) 
      $this->ModDefinition($DefName, $ArrayOfOptions);
    else 
      $this->AddDefinition($DefName, $ArrayOfOptions);

  } // function SetDefinition ()

  function ListDefinitions () {

    $CountOfLines = count($this->FileText);
    $ListIter = 0;
    for($iIter = 0; $iIter < $CountOfLines; $iIter++) {

      $CurLine = $this->FileText[$iIter];

      // Have we encountered a definition line?

      if (strncmp($CurLine, "[UBroker.WS.", 12) == 0) {
        $CurLine = str_replace("[UBroker.WS.", "", $CurLine);
        $CurLine = str_replace("]", "", $CurLine);
        $List[$ListIter] = $CurLine;
        $ListIter++;
      }

    }

    return $List;
  }

} // class MngUBrokerProperties

?>

